@@ -129,7 +129,7 @@ True
 >>> os.remove('/tmp/EICAR-éèô请收藏我们的网址')
 """
 
-__version__ = "0.3.16"
+__version__ = "0.3.17"
 
 
 # $Source$
@@ -140,6 +140,7 @@ import sys
 import socket
 import struct
 import base64
+import time
 
 ############################################################################
 
@@ -470,7 +471,7 @@ class _ClamdGeneric(object):
         else:
             # Python3
             assert isinstance(buffer_to_test, bytes) or isinstance(buffer_to_test, bytearray), 'Wrong type fom [buffer_to_test], should be bytes or bytearray [was {0}]'.format(type(buffer_to_test))
-            
+
         try:
             self._init_socket()
             self._send_command('INSTREAM')
@@ -478,10 +479,12 @@ class _ClamdGeneric(object):
         except socket.error:
             raise ConnectionError('Unable to scan stream')
 
-        max_chunk_size = 1024 # MUST be < StreamMaxLength in /etc/clamav/clamd.conf or /etc/clamd.conf
+        # MUST be < StreamMaxLength in /etc/clamav/clamd.conf
+        # or /etc/clamd.conf
+        max_chunk_size = 1024
 
         for n in range(1 + int(len(buffer_to_test)/max_chunk_size)):
-            chunk = buffer_to_test[n*max_chunk_size:(n+1)*max_chunk_size]            
+            chunk = buffer_to_test[n*max_chunk_size:(n+1)*max_chunk_size]
             size = struct.pack('!L', len(chunk))
             try:
                 self.clamd_socket.send(size)
@@ -492,9 +495,8 @@ class _ClamdGeneric(object):
             # Terminating stream
             self.clamd_socket.send(struct.pack('!L', 0))
 
-
         result='...'
-        dr={}
+        dr = {}
         while result:
             try:
                 result = self._recv_response()
@@ -502,15 +504,15 @@ class _ClamdGeneric(object):
                 raise ConnectionError('Unable to scan stream')
 
             if len(result) > 0:
-                
+
                 if result == 'INSTREAM size limit exceeded. ERROR':
                     raise BufferTooLongError(result)
 
                 filename, reason, status = self._parse_response(result)
-               
+
                 if status == 'ERROR':
                     dr[filename] = ('ERROR', '{0}'.format(reason))
-                    
+
                 elif status == 'FOUND':
                     dr[filename] = ('FOUND', '{0}'.format(reason))
 
@@ -536,13 +538,25 @@ class _ClamdGeneric(object):
         self.clamd_socket.send(cmd)
         return
 
-    
-
     def _recv_response(self):
         """
         receive response from clamd and strip all whitespace characters
         """
-        data = self.clamd_socket.recv(4096)
+        # If we connect too quickly
+        # sometimes we get a connexion error
+        # so we retry
+        failed_count = 5
+        while True:
+            try:
+                data = self.clamd_socket.recv(4096)
+            except socket.error:
+                time.sleep(0.01)
+                failed_count -= 1
+                if failed_count == 0:
+                    raise
+            else:
+                break
+
         try:
             response = bytes.decode(data).strip()
         except UnicodeDecodeError:
@@ -559,15 +573,7 @@ class _ClamdGeneric(object):
         response = ''
         c = '...'
         while c != '':
-            try:
-                data = self.clamd_socket.recv(4096)
-                try:
-                    c = bytes.decode(data).strip()
-                except UnicodeDecodeError:
-                    response = data.strip()
-            except socket.error:
-                break
-                
+            c = self._recv_response()
             response += '{0}\n'.format(c)
         return response
 
